
package cds;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicInteger;


public class CDS_03 {
    //constants to set up the desired test   
    static final int NUMTASKS=6;
    static final boolean SERIALEXEC = false; //either serial execution or via thread pooling
    static final int NUMTHREADS=3;
    static final boolean RUNNABLETEST=false;//true; //either runnableTest or callableTest
    
    //the executor and the array with tasks are kept in private fields
    private ExecutorService myExecutor;
    private MyTrivialTask[] myTasks;

    //the executor to be used is passed to the constructor
    public CDS_03() {
        this(Executors.newSingleThreadExecutor());
    }
    
    public CDS_03(ExecutorService exc) {
        myExecutor = exc;       
        // all the tasks are created
        myTasks = new MyTrivialTask[NUMTASKS];
        for (int i=0; i<myTasks.length; myTasks[i++]=new MyTrivialTask());  
    }
    

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        CDS_03 myInst;
        
        if(SERIALEXEC)
            myInst = new CDS_03();
        else
            myInst = new CDS_03(Executors.newFixedThreadPool(NUMTHREADS));

        if(RUNNABLETEST)
            myInst.doRunnableTest();
        else
            myInst.doCallableTest();        
        
       
    }
    
    private void doRunnableTest(){
        for(Runnable t: myTasks)
            myExecutor.execute(t);
        myExecutor.shutdown();
    }
    
    private void doCallableTest(){
        List<Future<String>> futureResults=new ArrayList<>();
        for(Callable c: myTasks)
            futureResults.add(myExecutor.submit(c));
                
        for(Future<String> f: futureResults) {
            try {
                System.out.println(f.get());
            } catch (InterruptedException | ExecutionException ex) {}
        }
        myExecutor.shutdown();
    }
   
    
}


// this class implements the body for a task (either runnable or callable)
class MyTrivialTask implements Runnable, Callable {
    
    static final int TASKDELAY=1000; //milliseconds
    static final int MAXNUM=10;
    static final AtomicInteger taskCounter = new AtomicInteger(0);
    final int myTaskID = taskCounter.getAndIncrement();

    @Override
    public void run() {
        System.out.println(call());
    }
    
    @Override
    public String call() {
        
        String message="";
        
        int myThreadID = ThreadSeqID.get(); //the ID for the thread running this task is obtained
        try {
            Thread.sleep(TASKDELAY);
        } catch (InterruptedException ex) {
            Thread.currentThread().interrupt();
        }
        int extractedNumber = new java.util.Random().nextInt(MAXNUM+1);
        
        for(int j=0;j<myTaskID;j++) {
            message+=".......";
        }
        message=message+"Task " + Integer.toString(myTaskID) + 
                " run by Thread "+ Integer.toString(myThreadID) + 
                " - # extracted: " + Integer.toString(extractedNumber);
        
        return message;
    }
    
}

