package cds;

import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.locks.*;

/**
 *
 * @author alessio
 */

public class ThreadStarvationDeadlockTest {
    static final int NUMTASKS=10;
    static final int POOLSIZE=4;
    static final int INITVALUE=-6; //initial number for the shared counter
    private final ExecutorService myExecutor;
    private final IncClass[] myTasks;
    
    public ThreadStarvationDeadlockTest(ExecutorService exc) {
        myExecutor = exc;       
        // all the tasks are created
        myTasks = new IncClass[NUMTASKS];
        for (int i=0; i<myTasks.length; myTasks[i++]=new IncClass());  
    }
    
    private void runAll(){
        for(Runnable t: myTasks)
            myExecutor.execute(t);
        myExecutor.shutdown();
    }
    
    public static void main(String[] arg){
        ThreadStarvationDeadlockTest myInst=
            new ThreadStarvationDeadlockTest(Executors.newFixedThreadPool(POOLSIZE));
        
        myInst.runAll();
    }
    


    static class IncClass implements Runnable{
        private static final Lock sharedLock = new ReentrantLock();
        private static final Condition cond = sharedLock.newCondition();
        private static final AtomicInteger c = new AtomicInteger(INITVALUE);

        public void IncClass(){
        }

        @Override
        public void run() {
            int id = c.getAndIncrement()-INITVALUE;
            
            //start message
            String msg="";
            for (int i=0; i++<id; msg+="  ");
            msg+="Task #"+Integer.toString(id)+": STARTED!";
            System.out.println(msg);
        
            sharedLock.lock();
            try {
                while(c.get()<0) {
                    cond.await();  //"sleeping", i.e. suspension on cond. queue
                }
                cond.signalAll();
            }
            catch (InterruptedException ie) {            
            }
            finally {
                sharedLock.unlock();
                
                //end message
                msg="";
                for (int i=0; i++<id; msg+="  ");
                msg+="Task #"+Integer.toString(id)+": DONE!";
                System.out.println(msg);
            }

        }
    }

}
